#pragma once

// C++ system level
#include <string>
#include <functional>

// ESP32 specific
// #include "freertos/FreeRTOS.h"
// #include "freertos/queue.h"

// project specific
#include "CyanBus.hpp"
#include "SpiderLib/NumberTypes.hpp"

// qthing stuff
#include <qthing>



namespace SiliconTorch {

  namespace FxCyanRGB8 {

    typedef struct __attribute__((__packed__)) {
      u8 r;
      u8 g;
      u8 b;
    } Pixel;

    extern const std::string HEADER;

    class FxCyanRGB8 : public qthing::Animation<qthing::RGB> {
      public:
        FxCyanRGB8(u16 leds, u16 startIdx = 0);

        void getAnimator();

        void registerAtCyanBus(CyanBus::CyanBus& cyanBus);

        // Register qthing UDP packet handler
        // mainly used for testing purposes
        void registerUDPHandler();

        // Strip the header before calling this function!
        void handlePacket(u8* buffer, std::size_t length);

        // qthing::Animator
        void step();
        qthing::RGB render(u16 addr);

      private:

        qthing::RGB* buffer0;
        qthing::RGB* buffer1;

        bool swapRequested = false;
        bool udpHandlerRegistered = false;

        u16 leds;
        u16 startIdx;


        void swapBuffers();

        void requestBufferSwap();

    };
  }
}
