#pragma once

// C++ system level
// #include <cstring>     // memset, strncmp
// #include <cstdlib>     // TODO: is this for memcpy?
// #include <functional>

// ESP32 specific
#include "esp_log.h"

// project specific
#include <Types.hpp>

// qthing stuff
// #include <qthing>


namespace SiliconTorch {

  namespace ServiceManager {

    extern const str NVSStateKey;

    enum NVSState : u8 {
      Disabled = 0,
      Enabled  = 1,
      Invalid
    };


    class Service {
      public:

        Service();


        // ╔ To be implemented by user! ╗

        virtual void init()  = 0;
        virtual void start() = 0;

        // ╚════════════════════════════╝


        const str& getName() const;
        const str& getNameSpace() const;

        u16 getStartOrder() const;

        bool isEnabled() const;
        void setEnabled(bool enabled);

        // internal stuff, only to be called by ServiceManager!
        void postInit();

      protected:

        void setName(const str& name);
        void setNameSpace(const str& nameSpace);
        void setStartOrder(u16 startOrder);

      private:

        NVSState nvsState = Disabled;

        bool attrsLocked = false;
        str name = "";
        str nameSpace = "";

        u16 startOrder = 1337;

    };


  }
}
